#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "lcd.h"

//PB0 = btn (pullup)
#define	BTN_BIT		(1<<0)
//PB1 = sensor
#define	SENSOR_BIT	(1<<1)
//PB2 = LED
#define	LED_BIT		(1<<2)

volatile uint8_t tmr1_overflow;

ISR(TIMER1_OVF_vect) {
	tmr1_overflow = 1;
}

int main() {
	char cnt_str[32];
	
	DDRB |= LED_BIT;
	PORTB |= BTN_BIT; //button pull-up
	
	lcd_init();
	
	lcd_write_string("Lagdisplay v1.0\n#### READY ####");
	
	// enable timer1 overflow interrupt
	TIMSK1 |= (1<<TOIE1);
	sei();
	
loop:
	while (!(PINB & BTN_BIT));
	//LED test
	while (PINB & BTN_BIT) {
		if (PINB & SENSOR_BIT)
			PORTB &= ~LED_BIT;
		else
			PORTB |= LED_BIT;			
	}
	while (!(PINB & BTN_BIT));
	lcd_clear();
	TCNT1 = 0;
	tmr1_overflow = 0;

wait_signal:
	while (!(ACSR & (1<<ACO)));
	
	TCCR1B = 0x03; //counter1 on, div64
	while (PINB & SENSOR_BIT);
	TCCR1B = 0x00; //counter1 off
	
	if (tmr1_overflow)
		snprintf(cnt_str, 32, "TIMER1 overflow");
	else
		snprintf(cnt_str, 32, "%lu us", ((uint32_t)TCNT1 << 2));
	
	lcd_write_string(cnt_str);
	
	goto loop;
}
