#include "lcd.h"

static void lcd_raw_out(uint8_t nibble) {
	//enable E, write LCD data bits
	LCD_PORT = (LCD_PORT & ~LCD_DATA_MASK) | LCD_E_BIT | (nibble << LCD_DATA_LSB_PIN);
	_delay_us(250);					//min 250us
	LCD_PORT &= ~LCD_E_BIT;			//disable E
	_delay_us(250);					//min 250us
}

static void lcd_write_op(uint8_t databyte) {
	LCD_PORT |= LCD_RS_BIT;			//set RS to 1
	_delay_us(1);					//min 20ns
	lcd_raw_out(databyte >> 4);		//high nibble
	lcd_raw_out(databyte & 0xF);	//low nibble
	_delay_us(40);					//min 40us
}

static void lcd_control_op(uint8_t databyte) {
	LCD_PORT &= ~LCD_RS_BIT;		//set RS to 0
	_delay_us(1);					//min 20ns
	lcd_raw_out(databyte >> 4);		//high nibble
	lcd_raw_out(databyte & 0xF);	//low nibble
	_delay_ms(2);					//min 2ms
}

//clear LCD
void lcd_clear(void) {
	//Clear
	lcd_control_op(0x01);
	//Return home (required with some displays)
	lcd_control_op(0x02);
}

//write a string to LCD
void lcd_write_string(const char* data) {
	uint8_t i = 0;

	lcd_clear();

	//write string char by char, change mem address to row2 if newline encountered
	while(data[i] != '\0') {
		if (data[i] == '\n')
			lcd_control_op(0xc0);
		else
			lcd_write_op((uint8_t)data[i]);
		i++;
	}
}

//initialize lcd (datasheet p.42)
void lcd_init(void) {
	LCD_DDR |= LCD_PORT_MASK;
	
	//wait for hw init (RS,E = 0)
	LCD_PORT &= ~LCD_PORT_MASK;
	_delay_ms(8);
	
	lcd_raw_out(0x2);		//func set -> 4-bit
	_delay_ms(8);
	//lcd_control_op(0x0a);	//lcd off
	lcd_control_op(0x28);	//func set -> 2L, 5x8
	lcd_control_op(0x0e);	//disp on
	lcd_control_op(0x06);	//entry mode set
}
